/* vi:set tabstop=2 shiftwidth=2 cindent: */
/*
 * libvxfs - library for reading Veritas Journaled FileSystem (VxFS)
 * Copyright (c) 1999 Martin Hinner <mhi@penguin.cz>
 *
 * fshead.c: Fileset header routines
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: fshead.c,v 0.4 1999/12/18 16:34:21 mhi Exp root $
 */
#include <stdio.h>
#include <stdlib.h>

#include <libvxfs.h>

int
vx_readfshead (struct vxfs *fs)
{
	struct vxfs_fsh *primary_fsh;
	struct vxfs_fsh *structural_fsh;
	struct vxfs_inode *fshino;

	/*
	 * Inodes are stored in extent(s) vxoltilist.iext[0,1]. Inode size is
	 * 0x100. 
	 */

	fshino = vxblkiget (fs, fs->iext, fs->fshino);

	structural_fsh = vxiread (fs, fshino, 0, 1);
	primary_fsh = vxiread (fs, fshino, 1, 1);


	if (vxdebug)
		{
			printf ("primary_fsh.version = %u\n", VXU32(fs, primary_fsh->version));
			printf ("primary_fsh.fsindex = %u\n", VXU32(fs, primary_fsh->fsindex));
			printf ("primary_fsh.iauino = %u\n", VXU32(fs, primary_fsh->iauino));
			printf ("primary_fsh.ninodes = %u\n", VXU32(fs, primary_fsh->ninodes));
			printf ("primary_fsh.maxinode = %u\n", VXU32(fs, primary_fsh->maxinode));
			printf ("primary_fsh.lctino = %u\n", VXU32(fs, primary_fsh->lctino));
			printf ("primary_fsh.nau = %u\n", VXU32(fs, primary_fsh->nau));
			printf ("primary_fsh.ilistino[0] = %u\n",
					VXU32(fs, primary_fsh->ilistino[0]));
			printf ("primary_fsh.ilistino[1] = %u\n",
					VXU32(fs, primary_fsh->ilistino[1]));
		}

	fs->structuralilistino = vxblkiget (fs, fs->iext,
																			VXU32(fs, structural_fsh->ilistino[0]));
	if (!VXFS_ISILT (fs->structuralilistino))
		{
			if (vxdebug)
				printf ("Warning: vxstructuralilistino is not type VXFS_FSILT!\n");
		}

	fs->ilistino = vxigetstructural (fs, VXU32(fs, primary_fsh->ilistino[0]));
	if (!VXFS_ISILT (fs->ilistino))
		{
			if (vxdebug)
				printf ("Warning: fs->ilistino is not type VXFS_FSILT!\n");
		}

	return 0;
}
