/* vi:set tabstop=2 cindent shiftwidth=2: */
/*
 * libvxfs - library for reading Veritas Journaled FileSystem (VxFS)
 * Copyright (c) 1999 Martin Hinner <mhi@penguin.cz>
 * Copyright (C) 1994, 1995, 1996 Theodore Ts'o.
 *
 * llseek.c: stub calling the llseek system call (from e2fsprogs)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: llseek.c,v 0.4 1999/12/18 16:34:21 mhi Exp root $
 */

#include "config.h"

#include <sys/types.h>

#include <errno.h>
#include <unistd.h>

#ifdef __linux__

#ifdef HAVE_LLSEEK
#include <unistd.h>
#include <syscall.h>

#else	/* HAVE_LLSEEK */

#ifdef __alpha__

#define llseek lseek

#else /* !__alpha__ */

#include <linux/unistd.h>

#ifndef __NR__llseek
#define __NR__llseek            140
#endif

static int _llseek (unsigned int, unsigned long,
		   unsigned long, loff_t *, unsigned int);

static _syscall5(int,_llseek,unsigned int,fd,unsigned long,offset_high,
		 unsigned long, offset_low,loff_t *,result,
		 unsigned int, origin)

static loff_t vxllseek (unsigned int fd, loff_t offset,
		unsigned int origin)
{
	loff_t result;
	int retval;

	retval = _llseek (fd, ((unsigned long long) offset) >> 32,
			((unsigned long long) offset) & 0xffffffff,
			&result, origin);
	return (retval == -1 ? (loff_t) retval : result);
}

#endif	/* HAVE_LLSEEK */

#endif /* __alpha__ */

loff_t vxllseek (unsigned int fd, loff_t offset,
			 unsigned int origin)
{
	loff_t result;
	static int do_compat = 0;

	if ((sizeof(off_t) >= sizeof(loff_t)) ||
	    (offset < ((loff_t) 1 << ((sizeof(off_t)*8) -1))))
		return lseek(fd, (off_t) offset, origin);

	if (do_compat) {
		errno = EINVAL;
		return -1;
	}
	
	result = llseek (fd, offset, origin);
	if (result == -1 && errno == ENOSYS) {
		/*
		 * Just in case this code runs on top of an old kernel
		 * which does not support the llseek system call
		 */
		do_compat++;
		errno = EINVAL;
	}
	return result;
}

#else /* !linux */

loff_t vxllseek (unsigned int fd, loff_t offset,
			 unsigned int origin)
{
	if ((sizeof(off_t) < sizeof(loff_t)) &&
	    (offset >= ((loff_t) 1 << ((sizeof(off_t)*8) -1)))) {
		errno = EINVAL;
		return -1;
	}
	return lseek (fd, (off_t) offset, origin);
}

#endif 	/* linux */


