/* vi:set tabstop=2 shiftwidth=2 cindent: */
/*
 * libvxfs - library for reading Veritas Journaled FileSystem (VxFS)
 * Copyright (c) 1999 Martin Hinner <mhi@penguin.cz>
 *
 * olt.c: Object Location Table routines
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: olt.c,v 0.4 1999/12/18 16:34:21 mhi Exp root $
 */
#include <stdio.h>
#include <stdlib.h>

#include <libvxfs.h>

int
vx_readolt (struct vxfs *fs)
{
	void *block;
	union vxfs_oltent *ent;
	struct vxfs_olt *olt;
	struct vxfs_oltilist *ilist;
	struct vxfs_oltfshead *oltfshead;

	if (vxdebug)
		printf ("Reading OLT (%u blocks at %08x)...\n",
						fs->oltsize, fs->oltext);

	block = vxread (fs, fs->oltext, fs->oltsize);
	olt = (struct vxfs_olt *)block;

	if (VXU32(fs,olt->magic) != VXFS_OLTMAGIC)
		{
			if (vxverbose)
				fprintf (stderr, "read_olt: Invalid magic number (vxolt.magic=%08x)\n",
								 VXU32(fs,olt->magic));
			return -1;
		}

	ent = block + olt->size;
	while ((void *)ent < (block + (fs->oltsize * fs->bsize)))
		{
			switch (VXU32(fs, ent->oltcommon.type))
				{
				case VXFS_OLTFSHEAD:
					oltfshead = (struct vxfs_oltfshead *)ent;
					break;
				case VXFS_OLTILIST:
					ilist = (struct vxfs_oltilist *)ent;
					break;
				}
			(int) ent += VXU32(fs, ent->oltcommon.size);
		}

	if (vxdebug)
		{
			printf ("ilist.iext[0] = %u\n", VXU32(fs, ilist->iext[0]));
			printf ("ilist.iext[1] = %u\n", VXU32(fs, ilist->iext[1]));
			printf ("oltfshead.fsino[0] = %u\n", VXU32(fs, oltfshead->fsino[0]));
			printf ("oltfshead.fsino[1] = %u\n", VXU32(fs, oltfshead->fsino[2]));
		}

	fs->iext = VXU32(fs, ilist->iext[0]);
	fs->fshino = VXU32(fs, oltfshead->fsino[0]);

	free (block);
	return 0;
}
